CREATED = 'created'
BUILDING = 'building'
UNSCHEDULABLE = 'unschedulable'
SCHEDULED = 'scheduled'
RUNNING = 'running'
SUCCEEDED = 'succeeded'
FAILED = 'failed'
STOPPED = 'stopped'
UNKNOWN = 'unknown'
